import 'package:flutter/services.dart';

/// Money input formatter supports money symbol, thousand separators and decimal separator.
class MoneyFormatter extends TextInputFormatter {
  late final String _decimalSeparator, _thousandSeparator;
  late final String? _symbol;

  MoneyFormatter(
      {required String decimalSeparator,
      String? symbol,
      required String thousandSeparator}) {
    _decimalSeparator = decimalSeparator;
    _symbol = symbol;
    _thousandSeparator = thousandSeparator;
  }

  /// Format input.
  String format(final String valueInString) {
    /// Replace all thousand separators then decimal separators.
    String string = valueInString.replaceAll(_thousandSeparator, '');

    /// Remove invalid decimals.
    final List<String> strings = string.split(_decimalSeparator);
    string =
        '${strings[0]}${strings.length > 1 ? '$_decimalSeparator${strings[1]}' : ''}';

    /// Remove symbol.
    if (_symbol != null) {
      string = string.replaceFirst(_symbol!, '');
    }

    final StringBuffer buffer = StringBuffer();

    /// Remove zero prefixs
    bool isZeroPrefix = true;
    for (int index = 0; index < string.length; index++) {
      if (string[index] == '0') {
        if (!isZeroPrefix) {
          buffer.write(string[index]);
        }
      } else {
        buffer.write(string[index]);
        if (isZeroPrefix) {
          isZeroPrefix = false;
        }
      }
    }
    string = buffer.toString();

    /// Write symbol at beginning.
    final StringBuffer formatted = StringBuffer(_symbol ?? '');

    int decimalDigits = 0; // digit amount after decimal separator.
    int calibratedIndex;
    final StringBuffer reversed = StringBuffer();

    /// Reverse value to formatting thousands.
    for (int index = string.length - 1; index >= 0; index--) {
      if (string.contains(_decimalSeparator) &&
          !reversed.toString().contains(_decimalSeparator)) {
        /// decimals
        decimalDigits++;
      } else {
        /// non decimals
        calibratedIndex = string.length - index - decimalDigits;
        if (calibratedIndex > 1 && calibratedIndex % 3 == 1) {
          reversed.write(_thousandSeparator);
        }
      }
      reversed.write(string[index]);
    }

    string = reversed.toString();

    /// Reverse back.
    for (int index = string.length - 1; index >= 0; index--) {
      formatted.write(string[index]);
    }

    return formatted.toString();
  }

  @override
  TextEditingValue formatEditUpdate(
      final TextEditingValue oldValue, final TextEditingValue newValue) {
    /// If cursor doesn't at the end then take the old value.
    final String value = newValue.selection.end == newValue.text.length
        ? newValue.text
        : oldValue.text;

    final String formatted = format(value);
    return TextEditingValue(
        selection: TextSelection.collapsed(offset: formatted.length),
        text: formatted);
  }
}
